% Crackle_Alignment.m
% We determine the fringe contrast pattern as a function of angular
% displacement. This will let us compare simulation and theoretical model
% developed.

%% Compile and create object
clear classes
MIST('Crackle_Michelson.mist');
m = Michelson(8); % The larger the no of modes used, the more accurate the 
% simulation

%% Set initial conditions
% Define global variables
global power_perror gainap gainsp N phi lower upper

% perturbation constants (x1 means misalignment along x axis of mirror 1)
perturbx1 = 0;
perturby1 = 0;
perturbx2 = 0;
perturby2 = 0;

power_perror = 0; % percentage power noise error
gainap = 1; % AP gain
gainsp = 1; % SP gain
N = 40; % number of points for fringe pattern
phi = linspace(0, pi, N); % generate linear points for phase propagation
lower = -0.003; % lower limit of range
upper = 0.003; % upper limit of range

%% Scan through x1 the fringe contrast

nptx1 = 250; %number of points for fringe contrast plot
% Scanning across angular range around perturbed misalignment
samplex1 = linspace(perturbx1+lower,perturbx1+upper, nptx1);
answerx1 = zeros(size(nptx1),2); % matrix to store plot data

% Determine fringe contrast pattern
for i=1:numel(samplex1)
    m.m1.tx = samplex1(i);
    answerx1(i,1)= m.m1.tx; % change x1 angular alignment
    answerx1(i,2)= f_contrast(m); % fringe contrast
end

figure()
plot(10^3*answerx1(:,1), answerx1(:,2), 'r--o')
xlabel('Angular misalignment of one mirror [mrad]', 'FontSize', 18)
ylabel('Fringe contrast', 'FontSize', 18)
hold all

%% Theoretical model (no misalignment, no noise, one dimensional)

k = 2*pi/m.lambda; % wavenumber
z = m.s2.L; % length of michelson arm
wo = 300e-6; % beam waist
% beam radius at beamsplitter
w = sqrt((wo)^2*(1+(m.lambda*z./(pi*(wo)^2))^2)); % beam radius
R = z*(1+((pi*wo^2)/(m.lambda*z))^2); % radius of curvature

modelx = linspace(-0.003, 0.003, 1000);
modely = exp(-2/(w^2)*modelx.^2*(z^2 + k^2*w^4/4 - k^2*z*w^4/(2*R) + ...
         k^2*z^2*w^4/(4*R^2)));
     
plot(10^3*modelx,modely, 'b')
legend('Simulation' , 'Expected')


%% Maximum gradient

% Calculate x-value of maximum gradient
maxpoint = sqrt(-1/(-4*z^2/w^2 -k^2*w^2 + 2*k^2*w^2*z/R - k^2*z^2*w^2/R^2));
% Maximum gradient
maxgrad = maxpoint*(-4*z^2/w^2 -k^2*w^2 + 2*k^2*w^2*z/R - k^2*z^2*w^2/R^2)*...
       (exp(-2/(w^2)*maxpoint^2*(z^2 + k^2*w^4/4 - k^2*z*w^4/(2*R) + ...
        k^2*z^2*w^4/(4*R^2))));
    
%% Relationship between gradient of graph and fringe contrast

for i=1:1000
modelgrad(i) = modelx(i)*(-4*z^2/w^2 -k^2*w^2 + 2*k^2*w^2*z/R - k^2*z^2*w^2/R^2)*...
       (exp(-2/(w^2)*modelx(i)^2*(z^2 + k^2*w^4/4 - k^2*z*w^4/(2*R) + ...
        k^2*z^2*w^4/(4*R^2))));
end

figure()
plot(modely, modelgrad)
xlabel('Fringe Contrast', 'FontSize', 18);
ylabel('Gradient of Fringe Contrast Pattern [rad]', 'FontSize', 18);
grid on
