% Crackle_Alignment_Gradient_Ascent_2.m has the same functionality as
% Crackle_Alignment_Gradient_Ascent. Except there is a smooth function is
% used to represent 'maximum step size', given by the inverse of the fringe
% contrast graph.
% I have experienced some oscillations using this approach, especially at
% fringe contrast = 0.6-0.7, possibly because gradient of graph at the
% region is approximately the same. Turns out this is not the most
% efficient approach, although theoretically it should be. Further
% investigation in this approach should be investigated.

%% Compile and create object
clear classes
MIST('Crackle_Michelson.mist');
m = Michelson(6);

%% Set initial conditions
% Define global variables
global power_perror gainap gainsp N phi

% perturbation constants (x1 means misalignment along x axis of mirror 1)
perturbx1 = 7e-4;
perturby1 = 6e-4;
perturbx2 = 9e-4;
perturby2 = 8e-4;

power_perror = 0; % percentage power noise error
gainap = 1; % AP gain
gainsp = 1; % SP gain
N = 40; % number of points for fringe pattern
phi = linspace(0, pi, N); % generate linear points for phase propagation
step = 1e-5; % unit step of angular displacement
i=1; % initiate number of iterations

%% First measurement of fringe contrast
m.m1.tx = perturbx1; % set perturbation on x1
m.m1.ty = perturby1; % set perturbation on y1
m.m2.tx = perturbx2; % set perturbation on x2
m.m2.ty = perturby2; % set perturbation on y2
fc = f_contrast(m); % measure initial fringe contrast
    
%% Iteration
x1_initial = m.m1.tx; % memory for x1 initial condition
y1_initial = m.m1.ty; % memory for y1 initial condition
x2_initial = m.m2.tx; % memory for x2 initial condition
y2_initial = m.m2.ty; % memory for y2 initial condition

alignment(1,i) = x1_initial; % x1 misalignment
alignment(2,i) = y1_initial; % y1 misalignment
alignment(3,i) = x2_initial; % x2 misalignment
alignment(4,i) = y2_initial; % y2 misalignment
alignment(5,i) = i; % loop number
alignment(6,i) = fc % fringe contrast after moving dof

tic
while fc<0.998 % threshold (arbitrary)
i=i+1; %increase step by 1
%% calculate X1 local gradient
m.m1.tx = x1_initial + step; % move in +ve direction
fcx1= f_contrast(m); % compute fringe contrast

if fcx1 < fc %if fringe contrast decreases
    m.m1.tx = x1_initial - step; %reverse direction
    fcx1 = f_contrast(m); % compute fringe contrast
end

gx1 = (fcx1-fc)/(m.m1.tx-x1_initial); % compute gradient
m.m1.tx = x1_initial; % return to original position, so it doesnt affect
% subsequent measurements of gradients
    
%% calculate Y1 local gradient
m.m1.ty = y1_initial + step; % move in +ve direction
fcy1= f_contrast(m); % compute fringe contrast

if fcy1 < fc %if fringe contrast decreases
    m.m1.ty = y1_initial - step; %reverse direction
    fcy1 = f_contrast(m); % compute fringe contrast
end

gy1 = (fcy1-fc)/(m.m1.ty-y1_initial); % compute gradient
m.m1.ty = y1_initial; % return to original position, so it doesnt affect
% subsequent measurements of gradients
    
%% calculate X2 local gradient
m.m2.tx = x2_initial + step; % move in +ve direction
fcx2= f_contrast(m); % compute fringe contrast

if fcx2 < fc %if fringe contrast decreases
    m.m2.tx = x2_initial - step; %reverse direction
    fcx2= f_contrast(m); % compute fringe contrast
end

gx2 = (fcx2-fc)/(m.m2.tx-x2_initial); % compute gradient
m.m2.tx = x2_initial; % return to original position, so it doesnt affect
% subsequent measurements of gradients
    
%% calculate Y2 local gradient
m.m2.ty = y2_initial + step; % move in +ve direction
fcy2= f_contrast(m); % compute fringe contrast

if fcy2 < fc %if fringe contrast decreases
    m.m2.ty = y2_initial - step; %reverse direction
    fcy2= f_contrast(m); % compute fringe contrast
end

gy2 = (fcy2-fc)/(m.m2.ty-y2_initial); % compute gradient
m.m2.ty = y2_initial; % return to original position

%% Determine dof with steepest gradient
 
gain = [gx1, gy1, gx2, gy2]; % vector of all gains

%% Theoretical model (no misalignment, no noise, one dimensional)

k = 2*pi/m.lambda; % wavenumber
z = m.s2.L; % length of michelson arm
wo = 300e-6; % beam waist
% beam radius at beamsplitter
w = sqrt((wo)^2*(1+(m.lambda*z./(pi*(wo)^2))^2)); % beam radius
R = z*(1+((pi*wo^2)/(m.lambda*z))^2); % radius of curvature

% Calculate x-value of maximum gradient
maxpoint = sqrt(-1/(-4*z^2/w^2 -k^2*w^2 + 2*k^2*w^2*z/R - k^2*z^2*w^2/R^2));
% Maximum gradient
maxgrad = abs(maxpoint*(-4*z^2/w^2 -k^2*w^2 + 2*k^2*w^2*z/R - k^2*z^2*w^2/R^2)*...
       (exp(-2/(w^2)*maxpoint^2*(z^2 + k^2*w^4/4 - k^2*z*w^4/(2*R) + ...
        k^2*z^2*w^4/(4*R^2)))));
    
% max step size
inv_fc_pattern =sqrt(log(fc)/((-2/(w^2))*(z^2 + k^2*w^4/4 - k^2*z*w^4/(2*R) + k^2*z^2*w^4/(4*R^2))));

%% Adjust step size multiple 'num', depending on fringe contrast measured
% crucial to balance between speed and possibility of stepping over the
% other end of the graph

move_step = inv_fc_pattern*gain/maxgrad;

%% Move mirror along dof with steepest gradient

a = [x1_initial, y1_initial, x2_initial, y2_initial];
a = a + move_step;

m.m1.tx = a(1);
x1_initial = m.m1.tx; % change initial condition in next loop
m.m1.ty = a(2);
y1_initial = m.m1.ty; % change initial condition in next loop
m.m2.tx = a(3);
x2_initial = m.m2.tx; % change initial condition in next loop
m.m2.ty = a(4);
y2_initial = m.m2.ty; % change initial condition in next loop

fc = f_contrast(m); % reset fringe contrast for next loop

%% Store Data

alignment(1,i) = x1_initial; % x1 misalignment
alignment(2,i) = y1_initial; % y1 misalignment
alignment(3,i) = x2_initial; % x2 misalignment
alignment(4,i) = y2_initial; % y2 misalignment
alignment(5,i) = i; % loop number
alignment(6,i) = fc % fringe contrast after moving dof

toc
end

% Plot graph of fringe contrast against number of steps
figure()
plot(alignment(5,:), alignment(6,:), 'b--o')
xlabel('Steps', 'FontSize', 14);
ylabel('Fringe Contrast', 'FontSize', 14);

figure()
plot(alignment(5,:), alignment(1,:)*10^3, 'g-o')
hold all
plot(alignment(5,:), alignment(2,:)*10^3, 'r-o')
hold all
plot(alignment(5,:), alignment(3,:)*10^3, 'b-x')
hold all
plot(alignment(5,:), alignment(4,:)*10^3, 'k-x')
hold all
xlabel('Steps', 'FontSize', 18);
ylabel('Angular displacement [m rad]', 'FontSize', 18);
legend('X1', 'Y1', 'X2', 'Y2');
grid on
