% Crackle_Frequency_Noise.m computes the relationship between coupling of
% laser frequency noise with the macroscopic length difference between
% Michelson arms. 

%% Compile and create object
clear classes
MIST('Crackle_Michelson.mist');
m = Michelson(6);
%% compute Michelson tranfer function of longitudinal motion of mirrors

% Modulation depth of differential motion of mirrors
m.z_d2_m1 = 1; % mirror 1
m.z_d2_m2 = -1; % mirror 2

fr = logspace(0,0.1,100); % Generate a set of logarithmic points
T = m.TransferFunction('d2', {'sy','asy'}, fr); % Transfer function

% calculate average (SP-AP) transfer function
% real to eliminate imaginary part that arises due to computation error 
tfmich = real(mean(T(1,:)-T(2,:))); 

% plot transfer function of AP and SP
figure()
subplot(211)
semilogx(fr, real(T(1,:)), fr, real(T(2,:)));
legend('SP', 'AP')
ylabel('Abs [W/m]', 'FontSize', 18)
grid
hold all
subplot(212)
semilogx(fr, angle(T(1,:)), fr, angle(T(2,:)));
legend('SP', 'AP');
grid
ylabel('Phase [rad]', 'FontSize', 18)
xlabel('Frequency [Hz]', 'FontSize', 18)

%% Laser frequency gain against macroscopic length difference

m.las.setMotionShape(0.5i); % imaginary value sets frequency modulation
m.z_d1_las = 1; % arbitrary modulation depth
m.f_d1 = 1000; % arbitrary modulation frequency

% Generate a set of points
pt = 100;
dL = linspace(0,0.0015, pt); % dL is half the macroscopic length difference
data = zeros(pt, 3); % just a matrix to store my data
tf1 = zeros(size(pt)); % transfer function calculated at asymmetric port
tf2 = zeros(size(pt)); % transfer function calculated at symmetric port

for k=1:pt
    % set the lengths (both arms are changed)
    m.s2.L = 1 + dL(k);
    m.s3.L = 1 - dL(k);
    % compute the optical TF
    tf1(k) = m.asy_TF_d1()/ (1j*m.f_d1); % divide TF by modulation freq
    tf2(k) = m.sy_TF_d1() / (1j*m.f_d1);
    data(k,1) = m.s2.L;
    data(k,2) = abs(tf1(k) - tf2(k))./tfmich; % gaind2 is calculated above
end

% plot TF difference against dL
figure()
plot (dL*1000, data(:,2)*10^18, 'o'); 
hold all
plot(dL*1000, dL*10^18 / (m.c_/m.lambda), 'r') % theoretical expression
ylabel('Gain of Frequency Noise Coupled \times 10^-^1^8 [m/Hz]',...
       'FontSize', 18)
xlabel('Macroscopic Length Difference [mm]', 'FontSize', 18)
legend('Simulation', 'Expected')
grid on
hold all