% Crackle_Transfer_Function.m computes the two transfer functions: one from
% the longitudinal motion of mirrors to the photodiodes, another from the
% laser frequency modulation to the photodiodes. 

%% Compile and create object
clear classes
MIST('Crackle_Michelson.mist');
m = Michelson(6);

%% compute Michelson tranfer function of moving mirrors

% Modulation depth of differential motion of mirrors
m.z_d2_m1 = 1;
m.z_d2_m2 = -1;

fr = logspace(0,3,100);
T = m.TransferFunction('d2', {'sy','asy'}, fr);

% real to eliminate imaginary part that arises due to computation error 
tfmich = real(mean(T(1,:)-T(2,:)));

% plot transfer function of AP and SP
figure()
subplot(211)
semilogx(fr, abs(T(1,:)*10^-7), fr, abs(T(2,:))*10^-7);
legend('SP', 'AP')
ylabel('Abs [10^7 W/m]',  'FontSize', 18)
grid
hold all
subplot(212)
semilogx(fr, angle(T(1,:)), fr, angle(T(2,:)));
legend('SP', 'AP');
grid
ylabel('Phase [rad]',  'FontSize', 18)
xlabel('Frequency [Hz]',  'FontSize', 18)

%% Compute transfer function of driving laser

% Phase modulation (real) power modulation (imaginary)
m.las.setMotionShape(0.5);
m.z_d1_las = 1;
% Set frequency of modulation
% m.f_d1 = 1e2;

% Compute transfer function
npt = 500;
fr = logspace(0,3,npt);
tflas1 = zeros(size(fr));
tflas2 = zeros(size(fr));
for i=1:npt
    % set the frequency
    m.f_d1 = fr(i);
    % compute the optical TF
    tflas1(i) = m.asy_TF_d1();
    tflas2(i) = m.sy_TF_d1();
end

% mean transfer function due to Michelson end mirrors
tfgain = real(mean(tflas2-tflas1));

% plot transfer functino of AP and SP
figure()
subplot(211)
semilogx(fr, abs(tflas2), fr, abs(tflas1))
legend('SP', 'AP')
ylabel('Abs[W/Hz]', 'FontSize', 18)
ylim([0,1])
grid
hold all
subplot(212)
semilogx(fr, angle(tflas2), fr, angle(tflas1))
legend('SP', 'AP');
grid
ylabel('Phase [rad]', 'FontSize', 18)
ylim([0,1])
xlabel('Frequency [Hz]','FontSize', 18)
